enum CommonRatios {
    _16x9,
    _16x10,
    _21x9,
    _43x18,
    _32x9,
    _4x3
}

string RatioString(CommonRatios ratio) {
    switch(ratio) {
        case _16x9:  return "16:9";
        case _16x10: return "16:10";
        case _21x9:  return "21:9";
        case _43x18: return "43:18";
        case _32x9:  return "32:9";
        case _4x3:   return "4:3";
        default:     return "bruh";
    }
}

[Setting hidden ]
CommonRatios s_commonRatio = _16x9;
[Setting hidden]
bool s_useCustomRatio = false;
[Setting hidden]
float s_customRatioX = 16.0;
[Setting hidden]
float s_customRatioY = 9.0;

[SettingsTab name="Ratio" icon="Desktop"]
void RenderSettings()
{
    s_useCustomRatio = UI::Checkbox("Custom Ratio", s_useCustomRatio);

    if(s_useCustomRatio) {
        UI::Text(Icons::ExclamationTriangle + " You can't just input a raw aspect ratio:");
        UI::Text("If you're going wider, make the second 9; if you're going taller, make the first 16");
        s_customRatioX = UI::InputFloat("Width", s_customRatioX);
        s_customRatioY = UI::InputFloat("Height", s_customRatioY);
        if (UI::Button("Apply")) {
            SquashHud();
        }
    } else {
        if (UI::BeginCombo("Ratio", RatioString(s_commonRatio))) {
            if(UI::Selectable("16:9", s_commonRatio == _16x9)){
                s_commonRatio = _16x9; SquashHud();
            }
            if(UI::Selectable("16:10", s_commonRatio == _16x10)){
                s_commonRatio = _16x10; SquashHud();
            }
            if(UI::Selectable("21:9", s_commonRatio == _21x9)){
                s_commonRatio = _21x9; SquashHud();
            }
            if (UI::IsItemHovered()) {

                UI::BeginTooltip();
                UI::Text("Most \"21:9\" displays (3440x1440, 2580x1080) are actually 43:18");
                UI::EndTooltip();
            }
            if(UI::Selectable("43:18", s_commonRatio == _43x18)){
                s_commonRatio = _43x18; SquashHud();
            }
            if(UI::Selectable("32:9", s_commonRatio == _32x9)){
                s_commonRatio = _32x9; SquashHud();
            }
            if(UI::Selectable("4:3", s_commonRatio == _4x3)){
                s_commonRatio = _4x3; SquashHud();
            }

            UI::EndCombo();
        }
    }
}

vec3 GetRatioVector() {
    if (s_useCustomRatio) {
        return vec3(s_customRatioX/10,s_customRatioY/10,1.0);
    }

    switch(s_commonRatio) {
        case _16x9:     return vec3( 1.6,0.9,1.0);
        case _16x10:    return vec3( 1.6,1.0,1.0);
        case _21x9:     return vec3(2.10,0.9,1.0);
        case _43x18:    return vec3(2.15,0.9,1.0);
        case _32x9:     return vec3(3.20,0.9,1.0);
        case _4x3:      return vec3(1.60,1.2,1.0);
        default:        return vec3( 1.6,0.9,1.0);
    }
}

bool HasHUD() {
    return (GetApp().CurrentPlayground !is null 
         && GetApp().CurrentPlayground.Interface !is null 
         && GetApp().CurrentPlayground.Interface.InterfaceRoot !is null);
}

MwFastBuffer<CHmsZoneOverlay@> overlayZones;

void SquashHud() {
    for(uint i = 0; i < overlayZones.Length; i++) {
        overlayZones[i].FurstrumHfDiag = GetRatioVector();
    }
    if (HasHUD()) {
        CControlContainer@ inter = GetApp().CurrentPlayground.Interface.InterfaceRoot;
        CHmsZoneOverlay@ zoneOverlay = cast<CHmsZoneOverlay@>(inter.Item.Corpus.Zone);
        zoneOverlay.FurstrumHfDiag = GetRatioVector();
    }
}

void SetupFrame(const string &in gbx) {
    CSystemFidFile@ gFid = Fids::GetGame(gbx);
    Fids::Preload(gFid);
    if (gFid is null) {
        print("Failed to correct frame: " + gbx);
        return;
    }
    CControlFrame@ frame = cast<CControlFrame@>(gFid.Nod);

    if (frame !is null 
    && (frame.Item !is null)
    && (frame.Item.Corpus !is null)
    && (frame.Item.Corpus.Zone !is null)) {
        CHmsZoneOverlay@ frameZone =  cast<CHmsZoneOverlay@>(frame.Item.Corpus.Zone);
        overlayZones.Add(frameZone);
        frameZone.FurstrumHfDiag = GetRatioVector();
    } else {
        print("Failed to correct frame: " + gbx);
    }
}

void SetupMenu(const string &in gbx) {
    CSystemFidFile@ gFid = Fids::GetGame(gbx);
    Fids::Preload(gFid);
    if (gFid is null) {
        print("Failed to correct menu: " + gbx);
        return;
    }
    CGameMenu@ gameMenu = cast<CGameMenu@>(gFid.Nod);
    CControlFrame@ frame = cast<CControlFrame@>(gameMenu.CurrentFrame);

    if (frame !is null 
    && (frame.Item !is null)
    && (frame.Item.Corpus !is null)
    && (frame.Item.Corpus.Zone !is null)) {
        CHmsZoneOverlay@ frameZone =  cast<CHmsZoneOverlay@>(frame.Item.Corpus.Zone);
        overlayZones.Add(frameZone);    
        frameZone.FurstrumHfDiag = GetRatioVector();
    } else {
        print("Failed to correct menu: " + gbx);
    }
}

void Main()
{
    CGameManiaPlanet@ game = cast<CGameManiaPlanet>(GetApp());
    SetupFrame("GameData/Menu/Control/Frame/FrameLoading.Frame.gbx");
    SetupFrame("GameData/Menu/Control/Frame/FrameMenuStations.Frame.Gbx");
    //SetupFrame("GameData/Menu/Control/Frame/FrameRaceCommon.Frame.gbx");
    //SetupFrame("GameData/Menu/Control/Frame/UiDriving2RootCommon.Frame.gbx");
    SetupFrame("GameData/Menu/Control/Frame/FrameManiaLinkBrowser.Frame.Gbx");
    SetupMenu("GameData/Menu/Menu/GameMenus.Menu.gbx");
    //SetupMenu("GameData/Menu/Menu/GameDialogs.Menu.gbx");
    //SetupMenu("GameData/Menu/Menu/GameSystemDialogs.Menu.gbx");

    MwFastBuffer<CHmsZoneOverlay@> overlays = game.Viewport.Overlays;
    overlayZones.Add(overlays[5]);
    overlays[5].FurstrumHfDiag = GetRatioVector(); // settings menu and chat

    // I can't reliably grab the hud from the fids, it's instantiated whenever

    while (game.LoadedManiaTitle is null) {
        yield();
    }
    bool foundHud = false;
    while(true) {
        if(HasHUD()) {
            if(!foundHud) {
                CControlContainer@ inter = game.CurrentPlayground.Interface.InterfaceRoot;
                CHmsZoneOverlay@ zoneOverlay = cast<CHmsZoneOverlay@>(inter.Item.Corpus.Zone);
                zoneOverlay.FurstrumHfDiag = GetRatioVector();
                foundHud = true;
            } else {
                yield();
            }
        } else {
            foundHud = false;
            yield();
        }
    }
}